#ifndef NRF905_H
#define NRF905_H

//------------------------------------------------------------------------
typedef enum{
    POWER_OFF = 1,
    SLEEP,
    STANDBY,
    REDUCED_RX,
    FULL_POWER
}t_NrfPwrMode;

//------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------
//  BYTE  | 7          | 6          | 5          |  4         | 3          | 2          | 1          | 0          |
//-----------------------------------------------------------------------------------------------------------------
//   0    | CH         | CH         | CH         | CH         | CH         | CH         | CH         | CH         |
//-----------------------------------------------------------------------------------------------------------------
//   1    | UNUSED     | UNUSED     | AUTO_R     | RX_RED     | PAPWR      | PAWWR      | PLL        | CH         |
//-----------------------------------------------------------------------------------------------------------------
//   2    | UNUSED     | TX_AFW     | TX_AFW     | TXAFW      | UNUSED     | RX_AFW     | RX_AFW     | RXAFW      |
//-----------------------------------------------------------------------------------------------------------------
//   3    | UNUSED     | UNUSED     | RX_PW      | RX_PW      | RX_PW      | RX_PW      | RX_PW      | RX_PW      |
//-----------------------------------------------------------------------------------------------------------------
//   4    | UNUSED     | UNUSED     | TX_PW      | TX_PW      | TX_PW      | TX_PW      | TX_PW      | TX_PW      |
//-----------------------------------------------------------------------------------------------------------------
//   5    | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS |
//-----------------------------------------------------------------------------------------------------------------
//   6    | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS |
//-----------------------------------------------------------------------------------------------------------------
//   7    | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS |
//-----------------------------------------------------------------------------------------------------------------
//   8    | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS | RX_ADDRESS |
//-----------------------------------------------------------------------------------------------------------------
//   9    | CRC_MODE   | CRC_EN     | XOF        | XOF        | XOF        | UP_CLK_EN  | UP_CLK_FREQ| UP_CLK_FREQ|
//-----------------------------------------------------------------------------------------------------------------

#define CONFIG_REG_LENGTH   10

// Frequency range 433.050 - 434.790MHz Channelspace = 100kHz
// f = 422.4 + CH_NO / 10

#define NRF_MAX_CH_NUMBER  16

#define NRF_CH_0_INIT      107	// 433.100 MHz
#define NRF_CH_1_INIT      108	// 433.200 MHz
#define NRF_CH_2_INIT      109	// 433.300 MHz
#define NRF_CH_3_INIT      110	// 433.400 MHz
#define NRF_CH_4_INIT      111	// 433.500 MHz
#define NRF_CH_5_INIT      112	// 433.600 MHz
#define NRF_CH_6_INIT      113	// 433.700 MHz
#define NRF_CH_7_INIT      114	// 433.800 MHz
#define NRF_CH_8_INIT      115	// 433.900 MHz
#define NRF_CH_9_INIT      116	// 434.000 MHz
#define NRF_CH_10_INIT     117	// 434.100 MHz 
#define NRF_CH_11_INIT     118	// 434.200 MHz 
#define NRF_CH_12_INIT     119	// 434.300 MHz
#define NRF_CH_13_INIT     120	// 434.400 MHz 
#define NRF_CH_14_INIT     121	// 434.500 MHz
#define NRF_CH_15_INIT     122	// 434.600 MHz



//------------------------------------------------------------------------


//------------------------------------------------------------------------
// NRF905 startup initialization
//------------------------------------------------------------------------
#define NRF_HFREQ_PLL_INIT      0x00    // init PLL Frequency
//  0x00 = 433MHz
//  0x01 = 868MHz, 915MHz
//------------------------------------------------------------------------
#define NRF_PA_PWR_INIT         0x03    // PA Output Power
//  0x00 =  -10dBm
//  0x01 =  -2dBm
//  0x02 =  +6dBm
//  0x03 =  +10dBm
//------------------------------------------------------------------------
#define NRF_RX_RED_PWR_INIT     0x00    // Rx Power
//  0x00 = Normal operation
//  0x01 = Reduced power
//------------------------------------------------------------------------
#define NRF_AUTO_RET_INIT       0x01    // Retransmit enable
//  0x00 = No retransmission
//  0x01 = Retransmission of data packages
//------------------------------------------------------------------------
#define NRF_RX_AFW_INIT         0x04    // Rx Address witdh
//  0x01 = 1 byte Rx address field width
//  0x02 = 2 byte Rx address field width
//  0x03 = 3 byte Rx address field width
//  0x04 = 4 byte Rx address field width
//------------------------------------------------------------------------
#define NRF_TX_AFW_INIT         0x04    // Tx Address witdh
//  0x01 = 1 byte Tx address field width
//  0x02 = 2 byte Tx address field width
//  0x03 = 3 byte Tx address field width
//  0x04 = 4 byte Tx address field width
//------------------------------------------------------------------------
#define NRF_RX_PW_INIT          0x20    // Rx Payload witdh
//  0x01 = 1 byte Rx payload field width
//  0x02 = 2 byte Rx payload field width
//  ...
//  0x20 = 32 byte Rx payload field width
//------------------------------------------------------------------------
#define NRF_TX_PW_INIT           0x20   // Tx Payload witdh
//  0x01 = 1 byte Tx payload field width
//  0x02 = 2 byte Tx payload field width
//  ...
//  0x20 = 32 byte Tx payload field width
//------------------------------------------------------------------------
#define NRF_RX_ADDRESS_INIT      0xFFFFFFFF         // Rx Address identity Byte 0
//------------------------------------------------------------------------
#define NRF_UP_CLK_FREQ_INIT     NRF_4MHZ_EXT_CLK   // Output clock frequency
#define NRF_4MHZ_EXT_CLK         0x00               // Output clock frequency = 4MHz
#define NRF_2MHZ_EXT_CLK         0x01               // Output clock frequency = 2MHz
#define NRF_1MHZ_EXT_CLK         0x02               // Output clock frequency = 1MHz
#define NRF_500KHZ_EXT_CLK       0x03               // Output clock frequency = 500kHz
//------------------------------------------------------------------------
#define NRF_UP_CLK_EN_INIT       NRF_UP_CLK_EN      // Output clock enable
#define NRF_UP_CLK_EN            0x01
#define NRF_UP_CLK_DIS           0x00
//  0x00 = No external clock signal available
//  0x01 = External clock enabled
//------------------------------------------------------------------------
#define NRF_XOF_INIT             0x03   // Crystal oscillator frequency
//  0x00 = 4MHz
//  0x01 = 8MHz
//  0x02 = 12MHz
//  0x03 = 16MHz
//  0x04 = 20MHz
//------------------------------------------------------------------------
#define NRF_CRC_EN_INIT          0x01   // CRC enable
//  0x00 = CRC disable
//  0x01 = CRC enable
//------------------------------------------------------------------------
#define NRF_CRC_MODE_INIT   	 0x01   // CRC mode
// 0x00 = 8 CRC check bit
// 0x01 = 16 CRC check bit
//------------------------------------------------------------------------

#define LOC_RX_ADDRESS      5
#define LOC_CHANNEL         0
#define RX_ADDRESS_SIZE     4
#define TX_ADDRESS_SIZE     4
#define CHANNEL_CONFIG_SIZE 2

// SPI basic instruction words
#define NRF_SPI_INSTR_W_CONFIG      0x00  // write configratuion register
#define NRF_SPI_INSTR_R_CONFIG      0x10  // read configration register
#define NRF_SPI_INSTR_W_TX_PAYLOAD  0x20  // write tx Payload 1-32 Bytes
#define NRF_SPI_INSTR_R_TX_PAYLOAD  0x21  // read tx Payload 1-32 Bytes
#define NRF_SPI_INSTR_W_TX_ADDRESS  0x22  // write tx Address
#define NRF_SPI_INSTR_R_TX_ADDRESS  0x23  // read tx Address
#define NRF_SPI_INSTR_R_RX_PAYLOAD  0x24  // read rx Payload 1- 32 Bytes
#define NRF_SPI_INSTR_CH_CONFIG     0x80  // fast ch no settings


//------------------------------------------------------------------------

typedef struct{
    unsigned int	Nrf905ChNo	    :  9;   // Channel Number
    unsigned int	Nrf905HfreqPll	    :  1;   // PLL Frequency
    unsigned int	Nrf905PaPwr	    :  2;   // PA Output Power
    unsigned int	Nrf905RxRedPwr	    :  1;   // Rx Power
    unsigned int	Nrf905AutoRetran    :  1;   // Retransmit enable
    unsigned int        /* UNUSED */        :  2;   // unused
    unsigned char	Nrf905RxAfw	    :  3;   // Rx Address witdh
    unsigned char       /* UNUSED */        :  1;   // unused
    unsigned char	Nrf905TxAfw	    :  3;   // Tx Address witdh
    unsigned char       /* UNUSED */        :  1;   // unused
    unsigned char	Nrf905RxPw	    :  6;   // Rx Payload witdh
    unsigned char       /* UNUSED */        :  2;   // unused
    unsigned char	Nrf905TxPw	    :  6;   // Tx Payload witdh
    unsigned char       /* UNUSED */        :  2;   // unused
    unsigned char	Nrf905RxAddress_0   :  8;   // Rx Address identity Byte 0
    unsigned char	Nrf905RxAddress_1   :  8;   // Rx Address identity Byte 1
    unsigned char	Nrf905RxAddress_2   :  8;   // Rx Address identity Byte 2
    unsigned char	Nrf905RxAddress_3   :  8;   // Rx Address identity Byte 3
    unsigned char	Nrf905UpClkFreq	    :  2;   // Output clock frequency
    unsigned char	Nrf905UpClkEn	    :  1;   // Output clock enable
    unsigned char       Nrf905Xof	    :  3;   // Crystal oscillator frequency
    unsigned char       Nrf905CrcEn	    :  1;   // CRC enable
    unsigned char	Nrf905CrcMode	    :  1;   // CRC mode
}t_NRF_Config_Bf;

typedef union{
    t_NRF_Config_Bf     uNrfConfigBf;
    unsigned char       aucNrfConfig[ CONFIG_REG_LENGTH ];

}t_NRF_Config_Reg;
//------------------------------------------------------------------------
typedef struct{
    unsigned char			    :  5;   // unused
    unsigned char	Nrf905Dr	    :  1;   // Data ready
    unsigned char			    :  1;   // unused
    unsigned char	Nrf905Am	    :  1;   // Address match
}t_NRF_StatRegBf;

typedef union{
    t_NRF_StatRegBf     uNrfStatRegBf;
    unsigned char       ucNrfStatReg;
}t_NRF_StatReg;
//------------------------------------------------------------------------
typedef struct{
    unsigned char TxAddress[NRF_TX_AFW_INIT];       // Tx Adress
    unsigned char TxPayLoad[NRF_TX_PW_INIT];	    // Tx Payload
    unsigned char RxPayLoad[NRF_RX_PW_INIT];        // Rx Payload 
}t_NRF_Buffer;
//------------------------------------------------------------------------
typedef struct{
    unsigned char ucNumber;
    unsigned char ucAssignement;    // duty cycle of assignement in %
}t_NRF_Channel;
//------------------------------------------------------------------------
typedef enum{
    WRITE_CONFIG = 1,
    WRITE_TX_PAYLOAD,
    WRITE_TX_ADDRESS,
    WRITE_RX_ADDRESS,
    WRITE_CHANNEL_CONFIG,
    READ_CONFIG,
    READ_TX_PAYLOAD,
    READ_TX_ADDRESS,
    READ_RX_PAYLOAD,
}t_spiInstructionSet;


//------------------------------------------------------------------------
extern t_NRF_Config_Reg     uNrfConfigReg;  // configration struct
extern t_NRF_StatReg        uNrfStatReg;    // status register
extern t_NRF_Buffer         uNrfBuffer;     // send, receiv and tx address buffer



//------------------------------------------------------------------------
// NRF905 Driver Prototypes
//------------------------------------------------------------------------
//------------------------------------------------------------------------
void NRF_SetPwrMode( unsigned char ucPowerMode );
void NRF_Init( void );
void NRF_SetChannel( unsigned int uiChannel );
void NRF_SetTxPwr( unsigned char uiTxPwr );
void NRF_SetRxAddress( unsigned long ulRxAddress );
void NRF_SetTxAddress( unsigned long ulTxAddress );
void NRF_GetTxAddress( void );
void NRF_ReadConfig( void );
void NRF_TxDownLoad( char* pTxStream, unsigned char ucStreamLength );
unsigned char NRF_RxUpLoad( char*pDestAddress );
void NRF_Send( void );
unsigned char NRF_CarrierSense( unsigned char ucChannelNumber, unsigned long ulScanTime );
t_NRF_Channel* getNrfChannel( void ); 
//------------------------------------------------------------------------

//------------------------------------------------------------------------

#endif

//------------------------------------------------------------------------
// END OF FILE
//------------------------------------------------------------------------
