#include <msp430x14x.h>
#include "timer_a.h"
#include "defs.h"

unsigned long ulTimeOut     = 0;        // timeout timer
unsigned char ucTimeOutEn   = FALSE;    // enable time out timer
unsigned long ulTimeMeas    = 0;        // time measure timer
unsigned char ucTimeMeasEn  = FALSE;    // enable time measure timer 

//------------------------------------------------------------------------
void InitTimerA( void )
{
    SET_BITS( TACTL, TASSEL1 + ID1 + TACLR );   // Timer A: Clear, Count up to CCR0, inputdivider = 4, Source = SMCLK
    SET_BITS( CCTL0, CCIE );                    // CCR0 interrupt enabled
    CCR0 = TIMER_A_TICK;                        // set timer tick
    SET_BITS( TACTL, MC0 );                     // Start Timer A in count up mode
}
//------------------------------------------------------------------------
unsigned long* SetTimeOut( unsigned long ulScanTime )
{
    ulTimeOut = ulScanTime;     // clear time out timer
    ucTimeOutEn = TRUE;         // enable timer
    return( &ulTimeOut );       // return address of timerout timer register
}

//------------------------------------------------------------------------
void ClearTimeMeasure( void )                          
{
    ucTimeMeasEn = FALSE;       // disable time measure timer 
    ulTimeMeas = 0;             // clear time measure timer
}
//------------------------------------------------------------------------
void StartTimeMeasure( void )
{
    ucTimeMeasEn = TRUE;        // enable time measure timer 
}
//------------------------------------------------------------------------
void StopTimeMeasure( void )
{
    ucTimeMeasEn = FALSE;       // disable time measure timer
}
//------------------------------------------------------------------------
unsigned long* ReadOutTimeMeasure( void )
{
    return( &ulTimeMeas );      // return pointer of timer result
}

//------------------------------------------------------------------------
#pragma vector=TIMERA0_VECTOR
__interrupt void timera_isr (void)
{
    CCR0 = TIMER_A_TICK;
    
    if( ucTimeOutEn ){
        if( !( ulTimeOut-- ) ){
            ucTimeOutEn = FALSE;
        }
    }
    
    if( ucTimeMeasEn ){
        ulTimeMeas++;
    }
}
//------------------------------------------------------------------------

