#include <msp430x14x.h>
#include "uart.h"
#include "defs.h"
#include "hwinit.h"
#include "datalink.h"


//------------------------------------------------------------------------
char acUart0RxBuffer[ UART0_RX_BUFFER_LENGTH ];
unsigned char ucRxBufferIndex = 0;

//------------------------------------------------------------------------
void ClearUart0RxBuffer( void );

//------------------------------------------------------------------------
void StartUart0( unsigned char Baudrate )
{
    ucRxBufferIndex = 0;                // clear rx buffer index     
    SET_BITS( P3SEL, UTXD0 + URXD0 );   // Enable USART0 TXD/RXD Ports
    SET_BITS( ME1, URXE0 );             // Enable USART0 RXD

    switch( Baudrate ){
        case BR_9600:   UBR00 = 0xA0; UBR10 = 0x01; UMCTL0 = 0x5B; break;
        case BR_19200:  UBR00 = 0xD0; UBR10 = 0x00; UMCTL0 = 0x11; break;
        case BR_115200: UBR00 = 0x22; UBR10 = 0x00; UMCTL0 = 0xDD; break;
    }
    SET_BITS( UCTL0, CHAR );                // 8-bit character
    SET_BITS( UTCTL0, SSEL0 + SSEL1 );      // UCLK = MCLK
    CLEAR_BITS( P2OUT, MAX3222_PWR_UP );    // Power up MAX3222 Device
    SET_BITS( P2OUT, MAX3222_SHDN );        // Disable shut down 
    CLEAR_BITS( UCTL0, SWRST );             // Initialize USART state machine
    SET_BITS( IE1, URXIE0 );                // Enable USART0 RX interrupt        
}

//------------------------------------------------------------------------
#pragma vector = USART0RX_VECTOR
__interrupt void UART0_rx (void)
{
    acUart0RxBuffer[ ucRxBufferIndex ] = RXBUF0;        // copy received character into the rx string
    if( ++ucRxBufferIndex < UART0_RX_BUFFER_LENGTH ){   // increment index and check range
        if( ( acUart0RxBuffer[ucRxBufferIndex -2 ] == 0x0D ) && ( acUart0RxBuffer[ucRxBufferIndex -1 ] == 0x0A ) ){ 
            //MacModem( (char*)&acUart0RxBuffer, ucRxBufferIndex );
            ClearUart0RxBuffer( );                      // clear uart0 rx buffer
        }
    }
    else    // buffer overflow -> clear buffer and buffer index
        ClearUart0RxBuffer( );
}
//------------------------------------------------------------------------
void ClearUart0RxBuffer( void )
{
    for( ucRxBufferIndex = 0; ucRxBufferIndex < UART0_RX_BUFFER_LENGTH; ucRxBufferIndex++ )
        acUart0RxBuffer[ucRxBufferIndex ] = 0;    
    ucRxBufferIndex = 0; 
}
//------------------------------------------------------------------------
void WriteUart0( char* pcSendStream, unsigned char ucStreamLength )
{
    SET_BITS( ME1, UTXE0 );             // Enable USART0 TXD module
    while( ucStreamLength ){
        while( !( IFG1 & UTXIFG0 ) );   // USART0 TX buffer ready?
        TXBUF0 = *pcSendStream;         // send next character
        pcSendStream++;
        ucStreamLength--;               // decrement stream length
    }
    while( !( IFG1 & UTXIFG0 ) );       // USART0 TX buffer ready?
    TXBUF0 = 0x0D;                      // Send CR
    while( !( IFG1 & UTXIFG0 ) );       // USART0 TX buffer ready?
    TXBUF0 = 0x0A;                      // Send LF
    CLEAR_BITS( ME1, UTXE0 );           // disable USART0 TXD module
}

